
/*
 * 
   

  Vektorkollektor plotter control / motor driver - reads serial commands, moves pen and draws
  =========================================================================================


  Protocol:

  BYTE 0: Start   (0)      /  start signal
  BYTE 1: Command (1-255)
  BYTE 2: X-POS   (1-200)
  BYTE 3: Y-POS   (1-220)
  BYTE 4: SERVO   (1-255)
  BYTE 5: CHKSM   (B0+B1+B2+B3+B4)%250+1 // CHKSM for command 4: (B0+B1+B2+B3+B4)%99+1 (workaround - as JS resolves int->char for values >= 128 as 2 bytes)

  Commands:

  1: move to coordinates, - draw if byte4 == 255, otherwise don't draw


  Connected hardware:

  - DC gearmotors (x&y)
  - End switches
  - Input comes from Arduino Nano / main- / joystick-controller

  Programmed with Arduino IDE 1.8.13

  ----------------------------------------------------------------------------
  "THE TSCHUNK-WARE LICENSE" (Revision 23):
  < nikl@s - roy.de > wrote this file. As long as you retain this notice you
  can do whatever you want with this stuff. If we meet some day, and you think
  this stuff is worth it, you can buy me a tschunk in return. Niklas
  ----------------------------------------------------------------------------
*/



#define XPWMA 5
#define XPWMB 3
#define YPWMA 11
#define YPWMB 8

#define XENCODER1 A7
#define XENCODER2 A6
#define XENDSWITCH 4
#define YENCODER1 A5
#define YENCODER2 A4
#define TEST_BUTTON 7
#define SERVO 2

#define Y_DOWN  digitalWrite(11,HIGH);digitalWrite(8,LOW);
#define Y_UP    digitalWrite(8,HIGH); digitalWrite(11,LOW);
#define Y_STOP  digitalWrite(8,LOW);  digitalWrite(11,LOW);

#define X_RIGHT    digitalWrite(5,HIGH); digitalWrite(3,LOW);
#define X_LEFT    digitalWrite(3,HIGH); digitalWrite(5,LOW);
#define X_STOP  digitalWrite(3,LOW);  digitalWrite(5,LOW);

#include <Servo.h>
Servo pen;

int xPos = 0;
int yPos = 0;
int up = 1;
int down = 2;
int penPos = down;

byte    receive[6];
byte    receiveIndex = 0;
boolean newMessageFlag = 0;
byte    command, xPosIn, yPosIn, servoIn, pServoIn;
int     chksm;

///////////////////////////////////////////////////////////// SETUP
void setup() {
  pinMode( XPWMA           , OUTPUT);
  pinMode( XPWMB           , OUTPUT);
  pinMode( XENCODER1       , INPUT);
  pinMode( XENCODER2       , INPUT);
  pinMode( XENDSWITCH      , INPUT_PULLUP);
  pinMode( YPWMA           , OUTPUT);
  pinMode( YPWMB           , OUTPUT);
  pinMode( YENCODER1       , INPUT);
  pinMode( YENCODER2       , INPUT);
  pinMode( TEST_BUTTON     , INPUT_PULLUP);
  pinMode( SERVO           , OUTPUT);
  digitalWrite(XPWMA, 0);
  digitalWrite(XPWMB, 0);
  digitalWrite(YPWMA, 0);
  digitalWrite(YPWMB, 0);
  Serial.begin(9600);
  pen.attach(SERVO);
  penUp();
  calibPosition();
}

///////////////////////////////////////////////////////////// LOOP
void loop() {

  pServoIn = servoIn;
  while (!receiveMessage()) {                  // wait for new message
    readPositions();                           // read encoders, update xPos and yPos
  }
  Serial.write(byte(6));                       // valid message recieved: respond with ACK
  
  if (command == 3) {
    calibPosition();
  }
  
  if (command == 1) {
    if (pServoIn != servoIn) {
      if (servoIn == 255) {
        penDown();
      } else {
        penUp();
      }
    }
    while (xPos != xPosIn || yPos != yPosIn) {
      readPositions();
      if (xPosIn > xPos) {
        X_RIGHT
      }
      if (xPosIn < xPos) {
        X_LEFT
      }
      if (xPosIn == xPos) {
        X_STOP
      }
      if (yPosIn > yPos) {
        Y_DOWN
      }
      if (yPosIn < yPos) {
        Y_UP
      }
      if (yPosIn == yPos) {
        Y_STOP
      }
    }
  }
}


void penDown() {
  if (penPos == up) {
    pen.write(10);
    delay(100);
    penPos = down;
  }
}

void penUp() {
  if (penPos == down) {
    pen.write(100);
    delay(100);
    penPos = up;
  }
}

void calibPosition() {
  // ..................................... calibrate left
  while (digitalRead(XENDSWITCH)) {
    X_LEFT
  }
  X_STOP

  xPos = 0;
  yPos = 0;
}
