/*
   VEKTORKOLLEKTOR Music-controller
   Runs on an Arduino UNO
   Programmed with Arduino 1.8.13
   
   Hardware connected to this controller is:
   - Arduino Nano with joystick / main control software
   - Adafruit MP3 shield
   
   ----------------------------------------------------------------------------
   "THE BEER-WARE LICENSE" (Revision 42):
   Niklas Roy wrote this file. As long as you retain this notice you
   can do whatever you want with this stuff. If we meet some day, and you think
   this stuff is worth it, you can buy me a beer in return :) Niklas
   ----------------------------------------------------------------------------
*/

// include SPI, MP3 and SD libraries
#include <SPI.h>
#include <Adafruit_VS1053.h>
#include <SD.h>

#define SHIELD_RESET  -1      // VS1053 reset pin (unused!)
#define SHIELD_CS     7      // VS1053 chip select pin (output)
#define SHIELD_DCS    6      // VS1053 Data/command select pin (output)

#define CARDCS 4     // Card chip select pin
#define DREQ 3       // VS1053 Data request, ideally an Interrupt pin

Adafruit_VS1053_FilePlayer musicPlayer =
  Adafruit_VS1053_FilePlayer(SHIELD_RESET, SHIELD_CS, SHIELD_DCS, DREQ, CARDCS);


int rndTrack = 1;
////

void setup() {
  Serial.begin(9600);
  musicPlayer.begin();
  musicPlayer.sineTest(0x44, 500);    // Make a tone to indicate VS1053 is working
  delay(50);
  musicPlayer.sineTest(0x88, 250);    // Make a tone to indicate VS1053 is working

  SD.begin(CARDCS);
  musicPlayer.setVolume(0, 0);
  musicPlayer.useInterrupt(VS1053_FILEPLAYER_PIN_INT);
  pinMode(A3,INPUT_PULLUP);

}

void loop() {
  playMp3(rndTrack);
  volumeWait();
}

void playMp3(int trackNumber) {
  Serial.println("....playing track# "+String(trackNumber));
  switch (trackNumber) {
    case 1: musicPlayer.startPlayingFile("1.MP3"); break;
    case 2: musicPlayer.startPlayingFile("2.MP3"); break;
    case 3: musicPlayer.startPlayingFile("3.MP3"); break;
    case 4: musicPlayer.startPlayingFile("4.MP3"); break;
    case 5: musicPlayer.startPlayingFile("5.MP3"); break;
    case 6: musicPlayer.startPlayingFile("6.MP3"); break;
    case 7: musicPlayer.startPlayingFile("7.MP3"); break;
    case 8: musicPlayer.startPlayingFile("8.MP3"); break;
    case 9: musicPlayer.startPlayingFile("9.MP3"); break;
    case 10: musicPlayer.startPlayingFile("10.MP3"); break;
    case 11: musicPlayer.startPlayingFile("11.MP3"); break;
    case 12: musicPlayer.startPlayingFile("12.MP3"); break;
    case 13: musicPlayer.startPlayingFile("13.MP3"); break;
    case 14: musicPlayer.startPlayingFile("14.MP3"); break;
    case 15: musicPlayer.startPlayingFile("15.MP3"); break;
    case 16: musicPlayer.startPlayingFile("16.MP3"); break;
    case 17: musicPlayer.startPlayingFile("17.MP3"); break;
    case 18: musicPlayer.startPlayingFile("18.MP3"); break;
    case 19: musicPlayer.startPlayingFile("19.MP3"); break;
    case 20: musicPlayer.startPlayingFile("20.MP3"); break;
    case 21: musicPlayer.startPlayingFile("21.MP3"); break;
    case 22: musicPlayer.startPlayingFile("20.MP3"); break;
    case 23: musicPlayer.startPlayingFile("21.MP3"); break;
  }
}


void volumeWait() {
  while (musicPlayer.playingMusic) { // set volume
    int volume = map(analogRead(2), 0, 1023, 100, 0);
    musicPlayer.setVolume(volume, volume);
    delay(10);
    rndTrack++;
    rndTrack=(rndTrack%21)+1;
  }
  int waitCount = 0;
  while (waitCount < 50) { // wait for start signal
    if (analogRead(A1) > 250 || digitalRead(A3) == false) {
      waitCount++;
    }
    delay(5);
    rndTrack++;
    rndTrack=(rndTrack%23)+1;
  }

}


/// File listing helper
void printDirectory(File dir, int numTabs) {
  while (true) {

    File entry =  dir.openNextFile();
    if (! entry) {
      // no more files
      //Serial.println("**nomorefiles**");
      break;
    }
    for (uint8_t i = 0; i < numTabs; i++) {
      Serial.print('\t');
    }
    Serial.print(entry.name());
    if (entry.isDirectory()) {
      Serial.println("/");
      printDirectory(entry, numTabs + 1);
    } else {
      // files have sizes, directories do not
      Serial.print("\t\t");
      Serial.println(entry.size(), DEC);
    }
    entry.close();
  }
}
